
from django.contrib.auth.decorators import permission_required
from django.core.paginator import Paginator, EmptyPage, PageNotAnInteger
from django.shortcuts import render, HttpResponseRedirect, get_object_or_404
from django.core.urlresolvers import reverse
from django.conf import settings
from django.db.models import Q
from django.contrib import messages
from django.utils.translation import ugettext as _

from axes.models import AccessAttempt
from axes import utils

from wagtailenforcer.forms.wagtailadmin import SearchForm


@permission_required('wagtailadmin.access_admin')
def list(request):
    """
    View to list all/filtered blocked login attempts
    """
    search_form = SearchForm(request.GET)
    is_searching = False

    # fetch blocked attempts, checking if overridden the failure limit
    FAILURE_LIMIT = getattr(settings, 'AXES_LOGIN_FAILURE_LIMIT', 3)
    attempts = AccessAttempt.objects.filter(failures_since_start__gte=FAILURE_LIMIT)

    if request.GET.get('search_field') and search_form.is_valid():
        search_text = search_form.cleaned_data.get('search_field')
        if search_text:
            attempts = attempts.filter(
                Q(username__icontains=search_text) |
                Q(user_agent__icontains=search_text)
            )
            is_searching = True

    attempts = attempts.order_by('-username')

    if attempts:
        p = request.GET.get('p', 1)
        paginator = Paginator(attempts, 20)

        try:
            attempts = paginator.page(p)
        except PageNotAnInteger:
            attempts = paginator.page(1)
        except EmptyPage:
            attempts = paginator.page(paginator.num_pages)

    return render(request, 'wagtailenforcer/attempts.html', {
        'attempts': attempts,
        'search_form': search_form,
        'is_searching': is_searching
    })


@permission_required('wagtailadmin.access_admin')
def reset(request, id):
    """
    View to reset axes AccessAttempt with given id
    """
    attempt = get_object_or_404(AccessAttempt, id=id)
    success = utils.reset(attempt.ip_address, attempt.username)

    user = attempt.username or attempt.ip_address

    if success > 0:
        messages.success(request, _("User '{0}' unblocked.").format(user))
    else:
        messages.error(request, _("User '{0}' was not blocked.").format(user))

    return HttpResponseRedirect(reverse('wagtailenforcer.views.axes.views.list'))



from django.core.urlresolvers import reverse

from wagtail.wagtailcore import hooks
from wagtail.wagtailadmin.menu import MenuItem


@hooks.register('register_settings_menu_item')
def register_blocked_users_item():
    return MenuItem('Blocked users', reverse('wagtailenforcer_blocked_users'), classnames='icon icon-locked', order=10000)



from django.conf.urls import url
from django.contrib.auth import views as django_auth_views

from wagtail.wagtailadmin.views import account

from axes.decorators import watch_login

from wagtailenforcer.forms.wagtailusers import PasswordForm, PasswordResetForm
from wagtailenforcer.views.wagtailusers import users

# Here we put all the overriden Wagtail urls from the different wagtail apps

urlpatterns = [
    url(
        r'^password_reset/$', account.password_reset, {
            'template_name': 'wagtailadmin/account/password_reset/form.html',
            'email_template_name': 'wagtailadmin/account/password_reset/email.txt',
            'subject_template_name': 'wagtailadmin/account/password_reset/email_subject.txt',
            'password_reset_form': PasswordResetForm,
            'post_reset_redirect': 'wagtailadmin_password_reset_done',
        }, name='wagtailadmin_password_reset'
    ),
    url(
        r'^password_reset/done/$', account.password_reset_done, {
            'template_name': 'wagtailadmin/account/password_reset/done.html'
        }, name='wagtailadmin_password_reset_done'
    ),
    url(
        r'^password_reset/confirm/(?P<uidb64>[0-9A-Za-z_\-]+)/(?P<token>[0-9A-Za-z]{1,13}-[0-9A-Za-z]{1,20})/$',
        account.password_reset_confirm, {
            'template_name': 'wagtailadmin/account/password_reset/enforce_confirm.html',
            'post_reset_redirect': 'wagtailadmin_password_reset_complete',
            'set_password_form': PasswordForm,
            'extra_context': {'reset_password_url': '/admin/password_reset'},
        }, name='wagtailadmin_password_reset_confirm',
    ),
    url(
        r'^password_reset/complete/$', account.password_reset_complete, {
            'template_name': 'wagtailadmin/account/password_reset/complete.html'
        }, name='wagtailadmin_password_reset_complete'
    ),

    url(r'^login/$', watch_login(account.login), name='wagtailadmin_login'),

    url(r'^users/(\d+)/$', users.edit, name='wagtailusers_users_edit'),
    url(r'^users/new/$', users.create, name='wagtailusers_users_create'),

]


# urlpatterns = [
#     url(
#         r'^$', account.password_reset, {
#             'template_name': 'wagtailadmin/account/password_reset/form.html',
#             'email_template_name': 'wagtailadmin/account/password_reset/email.txt',
#             'subject_template_name': 'wagtailadmin/account/password_reset/email_subject.txt',
#             'password_reset_form': PasswordResetForm,
#             'post_reset_redirect': 'wagtailadmin_password_reset_done',
#         }, name='wagtailadmin_password_reset'
#     ),
#     url(
#         r'^done/$', account.password_reset_done, {
#             'template_name': 'wagtailadmin/account/password_reset/done.html'
#         }, name='wagtailadmin_password_reset_done'
#     ),
#     url(
#         r'^confirm/(?P<uidb64>[0-9A-Za-z_\-]+)/(?P<token>[0-9A-Za-z]{1,13}-[0-9A-Za-z]{1,20})/$',
#         account.password_reset_confirm, {
#             'template_name': 'wagtailadmin/account/password_reset/confirm.html',
#             'post_reset_redirect': 'wagtailadmin_password_reset_complete',
#         }, name='wagtailadmin_password_reset_confirm',
#     ),
#     url(
#         r'^complete/$', account.password_reset_complete, {
#             'template_name': 'wagtailadmin/account/password_reset/complete.html'
#         }, name='wagtailadmin_password_reset_complete'
#     ),
# ]



from django import forms


class SearchForm(forms.Form):
    """
    Extremely simple form with only a search text input
    """
    search_field = forms.CharField(
        required=False,
        widget=forms.TextInput(attrs={'placeholder': 'Search Text'})
    )



from django.shortcuts import render, redirect, get_object_or_404
from django.contrib import messages
from django.contrib.auth import get_user_model
from django.utils.translation import ugettext as _
from django.contrib.auth.decorators import permission_required

from wagtail.wagtailcore.compat import AUTH_USER_APP_LABEL, AUTH_USER_MODEL_NAME

from wagtailenforcer.forms.wagtailusers import UserEditForm, UserCreationForm


User = get_user_model()

# Typically we would check the permission 'auth.change_user' for user
# management actions, but this may vary according to the AUTH_USER_MODEL
# setting
change_user_perm = "{0}.change_{1}".format(AUTH_USER_APP_LABEL, AUTH_USER_MODEL_NAME.lower())


@permission_required(change_user_perm)
def create(request):
    if request.POST:
        form = UserCreationForm(request.POST)
        if form.is_valid():
            user = form.save()
            messages.success(request, _("User '{0}' created.").format(user))
            return redirect('wagtailusers_users_index')
        else:
            messages.error(request, _("The user could not be created due to errors."))
    else:
        form = UserCreationForm()

    return render(request, 'wagtailusers/users/create.html', {
        'form': form,
    })


@permission_required(change_user_perm)
def edit(request, user_id):
    user = get_object_or_404(User, id=user_id)
    if request.POST:
        form = UserEditForm(request.POST, instance=user)
        if form.is_valid():
            user = form.save()
            messages.success(request, _("User '{0}' updated.").format(user))
            return redirect('wagtailusers_users_index')
        else:
            messages.error(request, _("The user could not be saved due to errors."))
    else:
        form = UserEditForm(instance=user)

    return render(request, 'wagtailusers/users/edit.html', {
        'user': user,
        'form': form,
    })



from django.db.models.signals import post_save, pre_save
from django.core.signals import got_request_exception
from django.dispatch import receiver
from wagtail.wagtaildocs.models import Document
from wagtail.wagtailimages.models import Image
from .utilities import check_for_virus


@receiver(pre_save, sender=Document)
def pre_file_save(sender, instance, **kwargs):
    instance = check_for_virus(instance)
    instance.clean()


@receiver(pre_save, sender=Image)
def pre_image_save(sender, instance, **kwargs):
    instance = check_for_virus(instance)
    instance.clean()





class WagtailenforcerException(Exception):
    def __init__(self, message):

        # Call the base class constructor with the parameters it needs
        super(WagtailenforcerException, self).__init__(message)


class EnforcerVirusException(WagtailenforcerException):
    pass


from __future__ import unicode_literals

from django import forms
from django.contrib.auth import get_user_model
from django.utils.translation import ugettext_lazy as _
from django.contrib.auth.forms import PasswordResetForm
from django.utils.translation import ugettext_lazy

from password_policies.conf import settings
from password_policies.forms import PasswordPoliciesForm
from password_policies.forms.fields import PasswordPoliciesField

from wagtail.wagtailusers.forms import UserCreationForm as BaseUserCreationForm

from wagtailenforcer.validators import UpperCaseLetterValidator

User = get_user_model()


class PasswordResetForm(PasswordResetForm):
    email = forms.EmailField(label=ugettext_lazy("Enter your email address to reset your password"), max_length=254)

    def clean(self):
        cleaned_data = super(PasswordResetForm, self).clean()

        # Find users of this email address
        UserModel = get_user_model()
        email = cleaned_data.get('email')
        if not email:
            raise forms.ValidationError(_("Please fill your email address."))
        active_users = UserModel._default_manager.filter(email__iexact=email, is_active=True)

        if active_users.exists():
            # Check if all users of the email address are LDAP users (and give an error if they are)
            found_non_ldap_user = False
            for user in active_users:
                if user.has_usable_password():
                    found_non_ldap_user = True
                    break

            if not found_non_ldap_user:
                # All found users are LDAP users, give error message
                raise forms.ValidationError(_("Sorry, you cannot reset your password here as your user account is managed by another server."))
        # else:
        #     # No user accounts exist
        #     raise forms.ValidationError(_("This email address is not recognised."))

        return cleaned_data


class PasswordForm(PasswordPoliciesForm):
    """
    Password form enforcing DEA password policy https://www.ict.govt.nz/guidance-and-resources/standards-compliance/authentication-standards/password-standard/6-password-minimum-requirements/
    """
    new_password1 = PasswordPoliciesField(
        label=_("New password"),
        max_length=settings.PASSWORD_MAX_LENGTH,
        min_length=settings.PASSWORD_MIN_LENGTH,
        validators=[UpperCaseLetterValidator()]
    )


class UserCreationForm(BaseUserCreationForm):

    password1 = PasswordPoliciesField(
        label=_("New password"),
        max_length=settings.PASSWORD_MAX_LENGTH,
        min_length=settings.PASSWORD_MIN_LENGTH,
        help_text=_("Leave blank if not changing."),
        validators=[UpperCaseLetterValidator()],
        required=False,
    )


class UserEditForm(forms.ModelForm):
    required_css_class = "required"

    error_messages = {
        'duplicate_username': _("A user with that username already exists."),
        'password_mismatch': _("The two password fields didn't match."),
    }
    username = forms.RegexField(
        label=_("Username"),
        max_length=30,
        regex=r'^[\w.@+-]+$',
        help_text=_("Required. 30 characters or fewer. Letters, digits and @/./+/-/_ only."),
        error_messages={
            'invalid': _("This value may contain only letters, numbers and @/./+/-/_ characters.")
        })

    email = forms.EmailField(required=True, label=_("Email"))
    first_name = forms.CharField(required=True, label=_("First Name"))
    last_name = forms.CharField(required=True, label=_("Last Name"))

    password1 = PasswordPoliciesField(
        label=_("New password"),
        max_length=settings.PASSWORD_MAX_LENGTH,
        min_length=settings.PASSWORD_MIN_LENGTH,
        help_text=_("Leave blank if not changing."),
        validators=[UpperCaseLetterValidator()],
        required=False,
    )
    password2 = forms.CharField(
        label=_("Password confirmation"), required=False,
        widget=forms.PasswordInput,
        help_text=_("Enter the same password as above, for verification."))

    is_superuser = forms.BooleanField(
        label=_("Administrator"),
        required=False,
        help_text=_("Administrators have the ability to manage user accounts.")
    )

    class Meta:
        model = User
        fields = ("username", "email", "first_name", "last_name", "is_active", "is_superuser", "groups")
        widgets = {
            'groups': forms.CheckboxSelectMultiple
        }

    def clean_username(self):
        # Since User.username is unique, this check is redundant,
        # but it sets a nicer error message than the ORM. See #13147.
        username = self.cleaned_data["username"]
        try:
            User._default_manager.exclude(id=self.instance.id).get(username=username)
        except User.DoesNotExist:
            return username
        raise forms.ValidationError(self.error_messages['duplicate_username'])

    def clean_password2(self):
        password1 = self.cleaned_data.get("password1")
        password2 = self.cleaned_data.get("password2")
        if password1 != password2:
            raise forms.ValidationError(
                self.error_messages['password_mismatch'])
        return password2

    def save(self, commit=True):
        user = super(UserEditForm, self).save(commit=False)

        # users can access django-admin if they are a superuser
        user.is_staff = user.is_superuser

        if self.cleaned_data["password1"]:
            user.set_password(self.cleaned_data["password1"])
        if commit:
            user.save()
            self.save_m2m()
        return user



#!/usr/bin/env python

import sys, os

try:
    from setuptools import setup, find_packages
except ImportError:
    from distutils.core import setup


# Hack to prevent "TypeError: 'NoneType' object is not callable" error
# in multiprocessing/util.py _exit_function when setup.py exits
# (see http://www.eby-sarna.com/pipermail/peak/2010-May/003357.html)
try:
    import multiprocessing
except ImportError:
    pass

PY3 = sys.version_info[0] == 3

install_requires = [
    'wagtail>=1.3',
    'django>=1.8',
    'django-password-policies',
    'django-axes',
    'pyclamd',
]

setup(
    name='wagtailenforcer',
    version='1.0.2',
    description='WagtailEnforcer, the Wagtail arm of the law.',
    author='Jordi J. Tablada, Dave Cartwright',
    author_email='jordi@springload.co.nz, dave@springload.co.nz',
    url='https://github.com/springload/wagtailenforcer/',
    packages=find_packages(),
    include_package_data=True,
    license='BSD',
    classifiers=[
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.2',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Framework :: Django',
    ],
    install_requires=install_requires,
    zip_safe=False,
)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals
import pyclamd

from django.utils.translation import ugettext_lazy as _
from django.conf import settings
from django.contrib import messages
from .exceptions import EnforcerVirusException


def check_for_virus(instance):
    has_virus, name = is_infected(instance.file.read())

    if has_virus:
        raise EnforcerVirusException(_('Virus "{}" was detected').format(name))

    return instance


def is_infected(stream):
    clam = get_clam()
    if getattr(settings, 'CLAMAV_ACTIVE', False) is False or clam is None:
        return None, ''

    result = clam.scan_stream(stream)
    if result:
        return True, result['stream'][1]

    return False, ''


def get_clam():
    try:
        clam = pyclamd.ClamdUnixSocket()

        # test if server is reachable
        clam.ping()

        return clam
    except pyclamd.ConnectionError:
        # if failed, test for network socket
        cd = pyclamd.ClamdNetworkSocket()
        try:
            cd.ping()
            return clam
        except pyclamd.ConnectionError:
            raise ValueError('could not connect to clamd server either by unix or network socket')



from django.conf.urls import patterns, url


# Here we put all the new Wagtail admin urls

urlpatterns = patterns(
    '',
    url(r'^$', 'wagtailenforcer.views.axes.views.list', name='wagtailenforcer_blocked_users'),
    url(r'^/reset/(\d+)/$', 'wagtailenforcer.views.axes.views.reset', name='wagtailenforcer_unblock_user'),
)



from django.utils.translation import ungettext

from password_policies.forms.validators import LetterCountValidator


class UpperCaseLetterValidator(LetterCountValidator):
    """
    Upper case validator
    """
    categories = ['Lu']

    def get_error_message(self):
        """
        Returns this validator's error message.
        """
        msg = ungettext("The new password must contain %d or more uppercase letter.",
                        "The new password must contain %d or more uppercase letter.",
                        self.get_min_count()) % self.get_min_count()
        return msg



from __future__ import unicode_literals
from django.contrib import messages
from django.http import JsonResponse

class WagtailenforcerMiddleware(object):

    """
    Provides full logging of requests and responses
    """
    _initial_http_body = None

    def process_request(self, request):
        self._initial_http_body = request.body # this requires because for some reasons there is no way to access request.body in the 'process_response' method.

    def process_exception(self, request, exception):
        """
        Catch raised validation exception
        """
        if type(exception).__name__ == 'EnforcerVirusException':
            return JsonResponse({
                'success': False,

                # https://github.com/django/django/blob/stable/1.6.x/django/forms/util.py#L45
                'error_message': exception.message,
            })
